@@ -96,8 +96,7 @@
   (condition-case nil (require 'ediff) (file-error))
   ;; Well, shouldn't Lazy Lock be as lazy as possible?
   ;(setq byte-compile-dynamic t byte-compile-dynamic-docstrings t)
-  ;; Shut Emacs' byte-compiler up (cf. stop me getting mail from users).
-  (setq byte-compile-warnings '(free-vars callargs redefine))
+  (defvar deactivate-mark)		; I think this is obsolete...
   ;;
   ;; We use this for clarity and speed.  Naughty but nice.
   (defmacro do-while (test &rest body)
@@ -211,7 +210,7 @@ To reduce machine load during stealth fontification, at the cost of stealth
 taking longer to fontify, you could increase the value of this variable.
 See also `lazy-lock-stealth-load'."
   :type '(choice (const :tag "never" nil)
-		 (number :tag "seconds"))	  
+		 (number :tag "seconds"))
   :group 'lazy-lock)
 
 (defcustom lazy-lock-stealth-verbose (not (null font-lock-verbose))
@@ -240,7 +239,7 @@ want to change this variable."
   :type 'boolean
   :group 'lazy-lock)
 
-(defcustom lazy-lock-invisible-foreground "gray50" 
+(defcustom lazy-lock-invisible-foreground "gray50"
   "The foreground colour to use to display invisible text.
 If nil, the default foreground is used.  If t, the default background is used.
 If a string, it should be a colour to use (either its name or its RGB value).
@@ -249,6 +248,13 @@ scrolling into unfontified regions."
   :type 'string
   :group 'lazy-lock)
 
+(defcustom lazy-lock-mode-line-string " Lazy"
+  "*String to display in the modeline when `lazy-lock-mode' is active.
+Set this to nil if you don't want a modeline indicator."
+  :type '(choice string
+		 (const :tag "none" nil))
+  :group 'lazy-lock)
+
 
 ;; User Functions:
 
@@ -456,14 +462,42 @@ See also variables `lazy-lock-walk-windows' and `lazy-lock-ignore-commands'."
 	    (and (eq lazy-lock-walk-windows t) (one-window-p t)))
        (or (window-minibuffer-p (selected-window))
 	    (lazy-lock-maybe-fontify-window (selected-window)))
-      (walk-windows #'lazy-lock-maybe-fontify-window
-		    'no-minibuf (eq lazy-lock-walk-windows 'all-frames)))))
+      (if (eq lazy-lock-walk-windows t)
+	  (lazy-lock-maybe-fontify-frame (selected-frame))
+	;; Visit all visible non-minibuffer-only frames on the selected device.
+	;; This is harder than it looks, since the `next-frame'
+	;; interface is error-prone - finding the starting frame is hard.
+	(catch 'lazy-lock-frame-loop-done
+	  (let* ((starting-frame (selected-frame))
+		 (frame starting-frame))
+	    (when (or (not (frame-visible-p frame))
+		      (frame-minibuffer-only-p frame))
+	      ;; starting-frame not suitable.
+	      (setq starting-frame (next-frame starting-frame 'visible-nomini))
+	      (when (eq starting-frame frame)
+		;; No suitable frames.
+		(throw 'lazy-lock-frame-loop-done t))
+	      (setq frame starting-frame))
+	    (while t
+	      (lazy-lock-maybe-fontify-frame frame)
+	      (setq frame (next-frame frame 'visible-nomini))
+	      (when (eq frame starting-frame)
+		(throw 'lazy-lock-frame-loop-done t)))))))))
+
+(defun lazy-lock-maybe-fontify-frame (frame)
+  ;; Fontify the given frame if we need to.  We first check the
+  ;; appropriate frame-modified-tick to avoid changing global state.
+  (let ((tick (frame-modified-tick frame)))
+    (unless (eq tick (frame-property frame 'lazy-lock-modified-tick))
+      (set-frame-property frame 'lazy-lock-modified-tick tick)
+      (with-selected-frame frame
+	(walk-windows #'lazy-lock-maybe-fontify-window 'no-minibuf)))))
 
 (defun lazy-lock-after-change-function (beg end old-len)
-  (and lazy-lock-mode
-       (if (= beg end)
-	   (font-lock-after-change-function beg end old-len)
-	 (lazy-lock-put-text-property beg end 'lazy-lock-fontified nil))))
+  (when lazy-lock-mode
+    (if (= beg end)
+	(font-lock-after-change-function beg end old-len)
+      (lazy-lock-put-text-property beg end 'lazy-lock-fontified nil))))
 
 (defvar lazy-lock-timeout-id nil)
 
@@ -494,21 +528,16 @@ See also variables `lazy-lock-walk-windows' and `lazy-lock-ignore-commands'."
 
 (defun lazy-lock-maybe-fontify-window (window)
   ;; Fontify the given window if we need to.  We first check the
-  ;; buffer-local value of lazy-lock-mode and the appropriate
-  ;; frame-modified-tick to make sure we should do the more accurate
-  ;; (but semi-expensive) checks in lazy-lock-fontify-window.  In this
-  ;; function, we are extremely careful not to change any global state
-  ;; (e.g. select-window, which will trip frame-modified-tick) until
-  ;; we've verified that we need to proceed to lazy-lock-fontify-window.
-  (let ((buffer (window-buffer window)))
-    (when (symbol-value-in-buffer 'lazy-lock-mode buffer)
-      (let* ((frame (window-frame window))
-	     (tick (frame-modified-tick frame)))
-	(unless (eq tick (frame-property frame 'lazy-lock-modified-tick))
-	  (set-frame-property frame 'lazy-lock-modified-tick tick)
-	  (save-selected-window
-	    (select-window window)
-	    (lazy-lock-fontify-window)))))))
+  ;; buffer-local value of lazy-lock-mode to make sure we should do
+  ;; the more accurate (but semi-expensive) checks in
+  ;; lazy-lock-fontify-window.  In this function, we are careful not
+  ;; to change any global state by calling select-window, which will
+  ;; trip frame-modified-tick, until we've verified that we need to
+  ;; proceed to lazy-lock-fontify-window.
+  (when (symbol-value-in-buffer 'lazy-lock-mode (window-buffer window))
+    (save-selected-window
+      (select-window window)
+      (lazy-lock-fontify-window))))
 
 (defun lazy-lock-fontify-window ()
   ;; Fontify the visible part of the buffer where necessary.
@@ -607,8 +636,8 @@ See also variables `lazy-lock-walk-windows' and `lazy-lock-ignore-commands'."
 
 (defun lazy-lock-fontify-walk-stealthily (ignored)
   ;; Loop over all buffers, fontify stealthily for each if necessary.
-  (let ((buffers (buffer-list)) (continue t) fontified message
-	message-log-max ;minibuffer-auto-raise
+  (let ((buffers (buffer-list)) (continue t) message
+	log-message-max-size ;minibuffer-auto-raise
 	)
     (save-excursion
       (do-while (and buffers continue)
@@ -658,7 +687,7 @@ See also variables `lazy-lock-walk-windows' and `lazy-lock-ignore-commands'."
 	      size (+ size (- end start))
 	      start end))
       ;; Saying "99% done" is probably better than "100% done" when it isn't.
-      (truncate (/ (* size 100.0) (buffer-size))))))
+      (truncate (if (> (buffer-size) 0) (/ (* size 100.0) (buffer-size)) 100)))))
 
 (defun lazy-lock-colour-invisible ()
   ;; Fontify the current buffer in `lazy-lock-invisible-face'.
@@ -705,7 +734,7 @@ See also variables `lazy-lock-walk-windows' and `lazy-lock-ignore-commands'."
 
 ;; XEmacs change: do it the right way.  This works with modeline mousing.
 ;;;###autoload
-(add-minor-mode 'lazy-lock-mode " Lazy")
+(add-minor-mode 'lazy-lock-mode 'lazy-lock-mode-line-string)
 
 ;; Provide ourselves:
 
