@@ -399,7 +399,7 @@ long after displaying each debugging message in mode line")
   ;;
   (or display-time-mail-arrived-file
       (setq display-time-mail-arrived-file
-	    (expand-file-name ".mail-arrived" (getenv "HOME"))))
+	    (expand-file-name ".mail-arrived" (user-home-directory))))
   )
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
